
from google_auth_oauthlib.flow import Flow
from colorama import init, Fore

SCOPES = ['https://www.googleapis.com/auth/youtube.upload']
API_SERVICE_NAME = 'youtube'
API_VERSION = 'v3'

VALID_PRIVACY_STATUSES = ('public', 'private', 'unlisted')





def get_credentials(secrets):
    flow = Flow.from_client_config(
        secrets,
        scopes=SCOPES,
        redirect_uri='urn:ietf:wg:oauth:2.0:oob')

    # Tell the user to go to the authorization URL.
    auth_url, _ = flow.authorization_url(prompt='consent')

    init(autoreset=True)
    print()
    print(Fore.GREEN + 'Please go to this URL: ' + Fore.CYAN + str(auth_url))

    # The user will get an authorization code. This code is used to get the
    # access token.
    print()
    code = input(Fore.GREEN + 'Enter the authorization code: ')
    flow.fetch_token(code=code)
    print()

    return flow.credentials

    # using flow.authorized_session.
    # session = flow.authorized_session()
    # print(session.get('https://www.googleapis.com/userinfo/v2/me').json())




import random
import time
from http import client
import httplib2

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload


# Explicitly tell the underlying HTTP transport library not to retry, since
# we are handling retry logic ourselves.
httplib2.RETRIES = 1

# Maximum number of times to retry before giving up.
MAX_RETRIES = 10

# Always retry when these exceptions are raised.
RETRIABLE_EXCEPTIONS = (httplib2.HttpLib2Error, IOError, client.NotConnected,
    client.IncompleteRead, client.ImproperConnectionState,
    client.CannotSendRequest, client.CannotSendHeader,
    client.ResponseNotReady, client.BadStatusLine)

# Always retry when an apiclient.errors.HttpError with one of these status
# codes is raised.
RETRIABLE_STATUS_CODES = [500, 502, 503, 504]

VALID_PRIVACY_STATUSES = ('public', 'private', 'unlisted')




def upload_video(
        credentials,
        file,
        title='video title',
        description='video sescription',
        category=22,
        privacy='private',
        tags=[]
    ):
    """
    youtube is made with get_authenticated_service()
    category is a number, see https://developers.google.com/youtube/v3/docs/videoCategories/list for more
    raises HttpError if connection error
    """
    youtube_service = build('youtube', 'v3', credentials=credentials)

    body=dict(
        snippet=dict(
            title=title,
            description=description,
            tags=tags,
            categoryId=category
        ),
        status=dict(
            privacyStatus=privacy
        )
    )

    # Call the API's videos.insert method to create and upload the video.
    insert_request = youtube_service.videos().insert(
        part=','.join(body.keys()),
        body=body,
        # The chunksize parameter specifies the size of each chunk of data, in
        # bytes, that will be uploaded at a time. Set a higher value for
        # reliable connections as fewer chunks lead to faster uploads. Set a lower
        # value for better recovery on less reliable connections.
        #
        # Setting 'chunksize' equal to -1 in the code below means that the entire
        # file will be uploaded in a single HTTP request. (If the upload fails,
        # it will still be retried where it left off.) This is usually a best
        # practice, but if you're using Python older than 2.6 or if you're
        # running on App Engine, you should set the chunksize to something like
        # 1024 * 1024 (1 megabyte).
        media_body=MediaFileUpload(file, chunksize=-1, resumable=True)
    )

    return resumable_upload(insert_request)

# This method implements an exponential backoff strategy to resume a
# failed upload.
def resumable_upload(request):
    response = None
    error = None
    retry = 0
    while response is None:
        try:
            print()
            print( 'Uploading file...')
            status, response = request.next_chunk()
            if response is not None:
                # print(response)
                if 'id' in response:
                    print()
                    print( 'Video id "%s" was successfully uploaded.' % response['id'])
                    return f'https://www.youtube.com/watch?v={response["id"]}'
                else:
                    exit('The upload failed with an unexpected response: %s' % response)
        except HttpError as e:
            if e.resp.status in RETRIABLE_STATUS_CODES:
                error = 'A retriable HTTP error %d occurred:\n%s' % (e.resp.status, e.content)
            else:
                raise
        except RETRIABLE_EXCEPTIONS as e:
            error = 'A retriable error occurred: %s' % e

        if error is not None:
            print( error)
            retry += 1
            if retry > MAX_RETRIES:
                exit('No longer attempting to retry.')

            max_sleep = 2 ** retry
            sleep_seconds = random.random() * max_sleep
            print( 'Sleeping %f seconds and then retrying...' % sleep_seconds)
            time.sleep(sleep_seconds)




import urllib
from urllib.request import urlopen
import json

URL = "https://www.googleapis.com/youtube/v3/videoCategories"

IDS = {
    "Film & Animation": 1,
    "Autos & Vehicles": 2,
    "Music": 10,
    "Pets & Animals": 15,
    "Sports": 17,
    "Short Movies": 18,
    "Travel & Events": 19,
    "Gaming": 20,
    "Videoblogging": 21,
    "People & Blogs": 22,
    "Comedy": 23,
    "Entertainment": 24,
    "News & Politics": 25,
    "Howto & Style": 26,
    "Education": 27,
    "Science & Technology": 28,
    "Nonprofits & Activism": 29,
    "Movies": 30,
    "Anime/Animation": 31,
    "Action/Adventure": 32,
    "Classics": 33,
    "Documentary": 35,
    "Drama": 36,
    "Family": 37,
    "Foreign": 38,
    "Horror": 39,
    "Sci-Fi/Fantasy": 40,
    "Thriller": 41,
    "Shorts": 42,
    "Shows": 43,
    "Trailers": 44,
}

def get_category_number(category):
    return IDS.get(category, 22)

def get(region_code="us", api_key=None):
    params = dict(part="snippet", regionCode=region_code, key=api_key)
    full_url = URL + "?" + urllib.urlencode(params)
    response = urlopen(full_url)
    categories_info = json.loads(response.read())
    items = categories_info["items"]
    return dict((item["snippet"]["title"], item["id"]) for item in items)



import sys
import yaml
import json
import traceback
from .upload_from_options import upload_from_options
from colorama import init, Fore


try:
    yaml_file = open(sys.argv[-1], 'r+')
    options = yaml.load(yaml_file.read())
    new_options = upload_from_options(options)
    yaml_file.close()
except KeyboardInterrupt:
    pass
except Exception as e:
    init(autoreset=True)
    print()
    print(Fore.RED + traceback.format_exc())
finally:
    print()
    yaml_file.close()




# yaml_file.seek(0)
# yaml_file.write(yaml.dump(
#         new_options,
#         indent=4,)
#     )
# yaml_file.truncate()




import json
import os
import requests
from .support import dotdict
from .upload_video import upload_video
# from .get_credentials import get_credentials
from google.oauth2.credentials import Credentials
from .get_category_number import get_category_number
from colorama import init, Fore
from pathlib import Path
from random import random
from .auth import default

SCOPES = ['https://www.googleapis.com/auth/youtube.upload']

def upload_from_options(options):
    """
    local_server: true

    secrets_path: |
        ...

    cresentials_path: |
        credentials

    videos:
        -
            title:  video title
            file:  path
            description: sdf
            category: 22
            privacy: private
            tags:
                - shit
                - holy
    """
    options = dotdict(**options)

    credentials = {}
    secrets = {}
    is_local = True

    if 'local_server' in options:
        is_local = options['local_server']

    if 'credentials' in options:
        credentials = Credentials.from_authorized_user_info(json.loads(options.credentials))

    elif 'credentials_path' in options and Path(options.credentials_path).exists():
        credentials = Credentials.from_authorized_user_file(options.credentials_path)

    elif Path('./credentials.json').exists():
        credentials = Credentials.from_authorized_user_file(options.credentials_path)

    elif 'secrets' in options:
        secrets = json.loads(options.secrets)

    elif 'secrets_path' in options and Path(options.secrets_path).exists():
        file = open(options.secrets_path, 'r')
        secrets = json.loads(file.read())
        file.close()

    if secrets.get('web'):
        print('''Warning: you chose a wrong api auth type,
        when creating a new ID client OAut
        you have to choose `other` and not `web application` in google cloud console:
        https://console.cloud.google.com/apis/credentials''')

    secrets = secrets.get('installed') or secrets.get('web')

    if not credentials and secrets:
        # credentials = get_credentials(secrets)
        credentials, _= default(
            SCOPES,
            client_id=secrets['client_id'],
            client_secret=secrets['client_secret'],
            auth_local_webserver=is_local
        )
        save_credentials(credentials, options.credentials_path or "./credentials.json")

    elif not credentials and not secrets:
        raise Exception('neither secrets or credentials')

    for video_options in options.videos:

        if 'category' in video_options:
            video_options['category'] = get_category_number(video_options['category'])

        if 'http' in video_options['file']:
            url = video_options['file']
            temp_file = './' + str(random())[2:] + '.mp4'
            temp_file = download_video(url, temp_file,)
            video_options['file'] = temp_file

        try:
            link = upload_video(
                credentials,
                **video_options
            )

        except Exception as e:
            init(autoreset=True)
            print()
            print(Fore.RED + str(e))

        finally:
            if 'temp_file' in vars() and temp_file:
                Path(temp_file).unlink()

    return link

def download_video(url, file_path):
        response = requests.get(url, stream=True)
        if response.status_code == 200:
                with open(file_path, 'wb') as f:
                    for chunk in response:
                        f.write(chunk)
        return str(Path(file_path).resolve())

def save_credentials(creds, credentials_path):

    creds_data = dump_credentials(creds)

    del creds_data['token']

    secrets_path = os.path.dirname(credentials_path)
    if not os.path.isdir(secrets_path):
        os.makedirs(secrets_path)

    with open(credentials_path, 'w') as outfile:
        json.dump(creds_data, outfile)


def dump_credentials(creds):
    creds_data = {
        'token': creds.token,
        'refresh_token': creds.refresh_token,
        'token_uri': creds.token_uri,
        'client_id': creds.client_id,
        'client_secret': creds.client_secret,
        'scopes': creds.scopes
    }
    return creds_data



class dotdict(dict):
    """dot.notation access to dictionary attributes"""
    def __getattr__(*args):
        val = dict.get(*args)
        return dict(val) if type(val) is dict else val
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__
